require "import"
import "android.content.Context"
import "android.net.wifi.WifiManager"
import "android.os.BatteryManager"
import "android.content.Intent"
import "android.content.IntentFilter"
import "android.telephony.TelephonyManager"
import "android.telephony.SubscriptionManager"
import "java.text.SimpleDateFormat"
import "java.util.Calendar"
import "android.net.ConnectivityManager"
import "java.util.Date"

local activity = service

-- Function to get current date and time
function getCurrentDateTime()
    local dateFormat = SimpleDateFormat("yyyy-MM-dd")
    local timeFormat = SimpleDateFormat("HH:mm:ss")
    local calendar = Calendar.getInstance()

    local currentDate = dateFormat.format(calendar.getTime())
    local currentTime = timeFormat.format(calendar.getTime())

    return currentDate, currentTime
end

-- Function to get current battery level
function getBatteryLevel()
    local batteryIntent = activity.registerReceiver(nil, IntentFilter(Intent.ACTION_BATTERY_CHANGED))
    local level = batteryIntent.getIntExtra(BatteryManager.EXTRA_LEVEL, -1)
    local scale = batteryIntent.getIntExtra(BatteryManager.EXTRA_SCALE, -1)

    local batteryLevel = level / scale * 100
    return string.format("%.0f%%", batteryLevel)
end

-- Function to get battery temperature
function getBatteryTemperature()
    local batteryIntent = activity.registerReceiver(nil, IntentFilter(Intent.ACTION_BATTERY_CHANGED))
    local temperature = batteryIntent.getIntExtra(BatteryManager.EXTRA_TEMPERATURE, 0) / 10
    return string.format("%.1f°C", temperature)
end

-- Function to get Wi-Fi signal strength and speed
function getWifiInfo()
    local wifiManager = activity.getSystemService(Context.WIFI_SERVICE)
    local wifiInfo = wifiManager.getConnectionInfo()

    if wifiInfo ~= nil then
        local signalLevel = WifiManager.calculateSignalLevel(wifiInfo.getRssi(), 5)
        local wifiSpeed = wifiInfo.getLinkSpeed() -- Speed in Mbps
        return string.format("%d/4", signalLevel), string.format("%d Mbps", wifiSpeed)
    else
        return "Not Connected", "N/A"
    end
end

-- Function to get mobile data signal strength and speed
function getMobileDataInfo()
    local telephonyManager = activity.getSystemService(Context.TELEPHONY_SERVICE)
    local signalStrength = telephonyManager.getSignalStrength()
    local networkType = telephonyManager.getNetworkType()

    local signalLevel = "N/A"
    local speed = "N/A"

    if signalStrength ~= nil then
        local levels = signalStrength.getLevel()
        signalLevel = string.format("%d/4", levels)
        
        -- Convert network type to speed (rough estimation)
        local speeds = {
            [13] = "4G LTE",
            [3] = "3G",
            [1] = "2G",
            [20] = "5G"
        }
        
        speed = speeds[networkType] or "Unknown"
    end

    return signalLevel, speed
end

-- Function to get SIM cards info (number of SIMs, operators, and signal strengths)
function getSimInfo()
    local subscriptionManager = activity.getSystemService(Context.TELEPHONY_SUBSCRIPTION_SERVICE)
    local subscriptionList = subscriptionManager.getActiveSubscriptionInfoList()

    local simInfo = ""

    for i = 0, subscriptionList.size() - 1 do
        local subscriptionInfo = subscriptionList.get(i)
        local carrierName = subscriptionInfo.getCarrierName()
        local simSlot = subscriptionInfo.getSimSlotIndex() + 1

        simInfo = simInfo .. string.format("SIM %d: %s\n", simSlot, carrierName)
    end

    return simInfo
end

-- Function to check mobile data status
function getMobileDataStatus()
    local connectivityManager = activity.getSystemService(Context.CONNECTIVITY_SERVICE)
    local mobileDataActive = connectivityManager.getNetworkInfo(ConnectivityManager.TYPE_MOBILE).isConnectedOrConnecting()

    if mobileDataActive then
        return "Mobile data is active."
    else
        return "Mobile data is not active."
    end
end

-- Function to get additional features
function getAdditionalFeatures()
    local tomorrowDate = os.date("%A, %B %d, %Y", os.time() + 86400)
    local dayOfWeek = os.date("%A")
    local endOfMonthDate = os.date("%d", os.time({year=os.date("%Y"), month=os.date("%m")+1, day=0}))
    local timeSinceEpoch = os.time()

    return tomorrowDate, dayOfWeek, endOfMonthDate, timeSinceEpoch
end

-- Function to speak date and time
function speakDateTime(context)
    local hour = tonumber(os.date("%H"))
    local greeting = ""
    
    if hour >= 5 and hour < 12 then
        greeting = "Good morning!"
    elseif hour >= 12 and hour < 18 then
        greeting = "Good afternoon!"
    elseif hour >= 18 or hour < 5 then
        greeting = "Good evening!"
    end

    local currentDate = os.date("%A, %B %d, %Y")
    local formattedTime = SimpleDateFormat("hh:mm a").format(Date())
    local batteryLevel = getBatteryLevel()
    local temperature = getBatteryTemperature()
    local mobileDataStatus = getMobileDataStatus()
    local tomorrowDate, dayOfWeek, endOfMonthDate, timeSinceEpoch = getAdditionalFeatures()

    local message = greeting .. " Today is " .. currentDate .. ". Tomorrow is " .. tomorrowDate .. ". It is " .. dayOfWeek .. ". The time is " .. formattedTime .. ". The battery level is " .. batteryLevel .. ". The battery temperature is " .. temperature .. ". " .. mobileDataStatus .. " This month ends on the " .. endOfMonthDate .. ". Current time since epoch is " .. timeSinceEpoch .. " seconds."
    context.speak(message)
end

-- Function to display information in separate dialogs
function showDialog(title, message)
    local dlg = LuaDialog(activity)
    dlg.setTitle(title)
    dlg.setMessage(message)
    dlg.setButton2("OK", function() dlg.dismiss() end)
    dlg.show()
end

-- Show each piece of information in a separate dialog
local currentDate, currentTime = getCurrentDateTime()
local batteryLevel = getBatteryLevel()
local batteryTemperature = getBatteryTemperature()
local wifiSignalStrength, wifiSpeed = getWifiInfo()
local mobileSignalStrength, mobileSpeed = getMobileDataInfo()
local simInfo = getSimInfo()
local mobileDataStatus = getMobileDataStatus()
local tomorrowDate, dayOfWeek, endOfMonthDate, timeSinceEpoch = getAdditionalFeatures()

-- Show dialogs for each task
showDialog("CSR with MK", "Current Date: " .. currentDate)
showDialog("CSR with MK", "Current Time: " .. currentTime)
showDialog("CSR with MK", "Battery Level: " .. batteryLevel)
showDialog("CSR with MK", "Battery Temperature: " .. batteryTemperature)
showDialog("CSR with MK", "Wi-Fi Signal Strength: " .. wifiSignalStrength)
showDialog("CSR with MK", "Wi-Fi Speed: " .. wifiSpeed)
showDialog("CSR with MK", "Mobile Signal Strength: " .. mobileSignalStrength)
showDialog("CSR with MK", "Mobile Speed: " .. mobileSpeed)
showDialog("CSR with MK", "SIM Info:\n" .. simInfo)
showDialog("CSR with MK", "Mobile Data Status: " .. mobileDataStatus)
showDialog("CSR with MK", "Additional Features:\nTomorrow's Date: " .. tomorrowDate .. "\nDay of Week: " .. dayOfWeek .. "\nEnd of Month Date: " .. endOfMonthDate .. "\nTime Since Epoch: " .. timeSinceEpoch .. " seconds")

-- Call the speakDateTime function
speakDateTime(activity)
service.playSoundClock()
return true
